@@ -10,10 +10,17 @@ from .tableformat import t
 
 def parse_args(args):
     parser = argparse.ArgumentParser(description="Inspect a schema")
+
     subparsers = parser.add_subparsers(help="sub-command help", dest="command")
 
     parser_deps = subparsers.add_parser("deps", help="Show inspected dependencies")
     parser_deps.add_argument("db_url", help="URL")
+
+    parser_deps2 = subparsers.add_parser(
+        "yaml", help="Export schema definition as YAML"
+    )
+    parser_deps2.add_argument("db_url", help="URL")
+
     return parser.parse_args(args)
 
 
@@ -31,8 +38,8 @@ def do_deps(db_url):
         )
 
         return dict(
-            thing=f"{dep.kind_dependent_on}: {thing}",
-            depends_on=f"{dep.kind}: {depends_on}",
+            thing="{}: {}".format(dep.kind_dependent_on, thing),
+            depends_on="{}: {}".format(dep.kind, depends_on),
         )
 
     deps = [process_row(_) for _ in deps]
@@ -45,10 +52,32 @@ def do_deps(db_url):
         print("No dependencies found.")
 
 
+def do_yaml(db_url):
+    with S(db_url) as s:
+        i = get_inspector(s)
+        defn = i.encodeable_definition()
+
+    from io import StringIO as sio
+
+    import yaml
+
+    x = sio()
+
+    yaml.safe_dump(defn, x)
+
+    print(x.getvalue())
+
+
 def run(args):
     if args.command == "deps":
         do_deps(args.db_url)
 
+    elif args.command == "yaml":
+        do_yaml(args.db_url)
+
+    else:
+        raise ValueError("no such commend")
+
 
 def do_command():  # pragma: no cover
     args = parse_args(sys.argv[1:])
