@@ -1,6 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import inspect
 import sys
 from collections import OrderedDict as od
 from itertools import groupby
@@ -279,7 +276,8 @@ class InspectedFunction(InspectedSelectable):
 
     @property
     def returntype_is_table(self):
-        return "." in self.returntype
+        if self.returntype:
+            return "." in self.returntype
 
     @property
     def signature(self):
@@ -1027,7 +1025,14 @@ PROPS = "schemas relations tables views functions selectables sequences constrai
 
 class PostgreSQL(DBInspector):
     def __init__(self, c, include_internal=False):
-        pg_version = c.dialect.server_version_info[0]
+        self.is_raw_psyco_connection = False
+
+        try:
+            pg_version = c.dialect.server_version_info[0]
+        except AttributeError:
+            pg_version = int(str(c.connection.server_version)[:-4])
+            self.is_raw_psyco_connection = True
+
         self.pg_version = pg_version
 
         def processed(q):
@@ -1037,7 +1042,12 @@ class PostgreSQL(DBInspector):
                 q = q.replace("-- 11_AND_LATER", "")
             else:
                 q = q.replace("-- 10_AND_EARLIER", "")
-            q = text(q)
+
+            if not self.is_raw_psyco_connection:
+                q = text(q)
+
+            else:
+                q = q.replace(r"\:", ":")
             return q
 
         if pg_version <= 9:
@@ -1072,12 +1082,21 @@ class PostgreSQL(DBInspector):
 
         super(PostgreSQL, self).__init__(c, include_internal)
 
+    def execute(self, *args, **kwargs):
+        result = self.c.execute(*args, **kwargs)
+
+        if result is None:
+            return self.c.fetchall()
+        else:
+            return result
+
     def load_all(self):
         self.load_schemas()
         self.load_all_relations()
         self.load_functions()
         self.selectables = od()
         self.selectables.update(self.relations)
+        self.selectables.update(self.composite_types)
         self.selectables.update(self.functions)
 
         self.load_privileges()
@@ -1091,7 +1110,7 @@ class PostgreSQL(DBInspector):
         self.load_deps_all()
 
     def load_schemas(self):
-        q = self.c.execute(self.SCHEMAS_QUERY)
+        q = self.execute(self.SCHEMAS_QUERY)
         schemas = [InspectedSchema(schema=each.schema) for each in q]
         self.schemas = od((schema.schema, schema) for schema in schemas)
 
@@ -1100,7 +1119,7 @@ class PostgreSQL(DBInspector):
             self.rlspolicies = od()
             return
 
-        q = self.c.execute(self.RLSPOLICIES_QUERY)
+        q = self.execute(self.RLSPOLICIES_QUERY)
 
         rlspolicies = [
             InspectedRowPolicy(
@@ -1119,7 +1138,7 @@ class PostgreSQL(DBInspector):
         self.rlspolicies = od((p.key, p) for p in rlspolicies)
 
     def load_collations(self):
-        q = self.c.execute(self.COLLATIONS_QUERY)
+        q = self.execute(self.COLLATIONS_QUERY)
         collations = [
             InspectedCollation(
                 schema=i.schema,
@@ -1135,7 +1154,7 @@ class PostgreSQL(DBInspector):
         self.collations = od((i.quoted_full_name, i) for i in collations)
 
     def load_privileges(self):
-        q = self.c.execute(self.PRIVILEGES_QUERY)
+        q = self.execute(self.PRIVILEGES_QUERY)
         privileges = [
             InspectedPrivilege(
                 object_type=i.object_type,
@@ -1149,7 +1168,7 @@ class PostgreSQL(DBInspector):
         self.privileges = od((i.key, i) for i in privileges)
 
     def load_deps(self):
-        q = self.c.execute(self.DEPS_QUERY)
+        q = self.execute(self.DEPS_QUERY)
 
         self.deps = list(q)
 
@@ -1305,7 +1324,7 @@ class PostgreSQL(DBInspector):
         self.materialized_views = od()
         self.composite_types = od()
 
-        q = self.c.execute(self.ENUMS_QUERY)
+        q = self.execute(self.ENUMS_QUERY)
         enumlist = [
             InspectedEnum(
                 name=i.name,
@@ -1316,7 +1335,7 @@ class PostgreSQL(DBInspector):
             for i in q
         ]
         self.enums = od((i.quoted_full_name, i) for i in enumlist)
-        q = self.c.execute(self.ALL_RELATIONS_QUERY)
+        q = self.execute(self.ALL_RELATIONS_QUERY)
 
         for _, g in groupby(q, lambda x: (x.relationtype, x.schema, x.name)):
             clist = list(g)
@@ -1383,7 +1402,7 @@ class PostgreSQL(DBInspector):
         self.relations = od()
         for x in (self.tables, self.views, self.materialized_views):
             self.relations.update(x)
-        q = self.c.execute(self.INDEXES_QUERY)
+        q = self.execute(self.INDEXES_QUERY)
         indexlist = [
             InspectedIndex(
                 name=i.name,
@@ -1408,7 +1427,7 @@ class PostgreSQL(DBInspector):
             for i in q
         ]
         self.indexes = od((i.quoted_full_name, i) for i in indexlist)
-        q = self.c.execute(self.SEQUENCES_QUERY)
+        q = self.execute(self.SEQUENCES_QUERY)
 
         sequencelist = [
             InspectedSequence(
@@ -1420,7 +1439,7 @@ class PostgreSQL(DBInspector):
             for i in q
         ]
         self.sequences = od((i.quoted_full_name, i) for i in sequencelist)
-        q = self.c.execute(self.CONSTRAINTS_QUERY)
+        q = self.execute(self.CONSTRAINTS_QUERY)
 
         constraintlist = []
 
@@ -1431,7 +1450,7 @@ class PostgreSQL(DBInspector):
                 constraint_type=i.constraint_type,
                 table_name=i.table_name,
                 definition=i.definition,
-                index=i["index"],
+                index=getattr(i, "index"),
                 is_fk=i.is_fk,
                 is_deferrable=i.is_deferrable,
                 initially_deferred=i.initially_deferred,
@@ -1453,7 +1472,7 @@ class PostgreSQL(DBInspector):
 
         self.constraints = od((i.quoted_full_name, i) for i in constraintlist)
 
-        q = self.c.execute(self.EXTENSIONS_QUERY)
+        q = self.execute(self.EXTENSIONS_QUERY)
         extensionlist = [
             InspectedExtension(name=i.name, schema=i.schema, version=i.version)
             for i in q
@@ -1472,7 +1491,7 @@ class PostgreSQL(DBInspector):
 
     def load_functions(self):
         self.functions = od()
-        q = self.c.execute(self.FUNCTIONS_QUERY)
+        q = self.execute(self.FUNCTIONS_QUERY)
         for _, g in groupby(q, lambda x: (x.schema, x.name, x.identity_arguments)):
             clist = list(g)
             f = clist[0]
@@ -1535,7 +1554,7 @@ class PostgreSQL(DBInspector):
             self.functions[s.quoted_full_name + identity_arguments] = s
 
     def load_triggers(self):
-        q = self.c.execute(self.TRIGGERS_QUERY)
+        q = self.execute(self.TRIGGERS_QUERY)
         triggers = [
             InspectedTrigger(
                 i.name,
@@ -1551,7 +1570,7 @@ class PostgreSQL(DBInspector):
         self.triggers = od((t.signature, t) for t in triggers)
 
     def load_types(self):
-        q = self.c.execute(self.TYPES_QUERY)
+        q = self.execute(self.TYPES_QUERY)
 
         def col(defn):
             return defn["attribute"], defn["type"]
@@ -1562,7 +1581,7 @@ class PostgreSQL(DBInspector):
         self.types = od((t.signature, t) for t in types)
 
     def load_domains(self):
-        q = self.c.execute(self.DOMAINS_QUERY)
+        q = self.execute(self.DOMAINS_QUERY)
 
         def col(defn):
             return defn["attribute"], defn["type"]
@@ -1605,18 +1624,34 @@ class PostgreSQL(DBInspector):
             setattr(self, prop, filtered)
 
     def _as_dicts(self):
-        def obj_to_d(x):
+        done = set()
+
+        def obj_to_d(x, k=None):
+            if id(x) in done:
+                if isinstance(x, (str, bool, int)):
+                    return x
+                elif hasattr(x, "quoted_full_name"):
+                    return x.quoted_full_name
+                return "..."
+            done.add(id(x))
+
             if isinstance(x, dict):
-                return {k: obj_to_d(v) for k, v in x.items()}
+                return {k: obj_to_d(v, k) for k, v in x.items()}
 
             elif isinstance(x, (ColumnInfo, Inspected)):
+
+                def safe_getattr(x, k):
+                    try:
+                        return getattr(x, k)
+                    except NotImplementedError:
+                        return "NOT IMPLEMENTED"
+
                 return {
-                    k: obj_to_d(getattr(x, k))
+                    k: obj_to_d(safe_getattr(x, k), k)
                     for k in dir(x)
-                    if not k.startswith("_") and not inspect.ismethod(getattr(x, k))
+                    if not k.startswith("_") and not callable(safe_getattr(x, k))
                 }
             else:
-
                 return str(x)
 
         d = {}
@@ -1630,6 +1665,20 @@ class PostgreSQL(DBInspector):
 
         return d
 
+    def encodeable_definition(self):
+        return self._as_dicts()
+
+    def as_yaml(self):
+        from io import StringIO as sio
+
+        import yaml
+
+        s = sio()
+
+        yaml.safe_dump(self.encodeable_definition(), s)
+
+        return s.getvalue()
+
     def one_schema(self, schema):
         self.filter_schema(schema=schema)
 
